/*******************************************************************************
 * Copyright (c) 2017, 2018 fortiss GmbH. 
 * 
 * This program and the accompanying materials are made available under the
 * terms of the Eclipse Public License v2.0 which is available at
 * http://www.eclipse.org/legal/epl-v20.html
 * 
 * SPDX-License-Identifier: EPL-2.0
 * 
 * Contributors:
 *     Florian Hoelzl (fortiss GmbH) - initial implementation
 *
 *******************************************************************************/
package org.fortiss.tooling.base.ui.editor.fx;

import static java.lang.Math.max;

import java.util.Collections;
import java.util.LinkedList;
import java.util.List;

import org.eclipse.emf.ecore.EObject;
import org.eclipse.systemfocus.kernel.common.ui.javafx.lwfxef.DiagramCoordinate;
import org.eclipse.systemfocus.kernel.common.ui.javafx.lwfxef.DiagramViewerFeatures;
import org.eclipse.systemfocus.kernel.common.ui.javafx.lwfxef.controller.base.ControllerBase;
import org.eclipse.systemfocus.kernel.common.ui.javafx.lwfxef.model.IModelChangeListener;
import org.eclipse.systemfocus.kernel.common.ui.javafx.lwfxef.model.IModelChangeProvider;
import org.eclipse.systemfocus.kernel.common.ui.javafx.lwfxef.mvc.IMVCBundle;
import org.fortiss.tooling.kernel.service.IElementCompositorService;
import org.fortiss.tooling.kernel.ui.service.IContextMenuService;

import javafx.scene.Node;
import javafx.scene.control.MenuItem;

/**
 * Diagram controller that uses the {@link IElementCompositorService} to create context menu
 * entries for prototypes provided by the {@link IContextMenuService}.
 */
public abstract class EObjectBasedDiagramControllerBase extends ControllerBase
		implements IModelChangeProvider {
	/** Constructor. */
	public EObjectBasedDiagramControllerBase(IMVCBundle mvcb) {
		super(mvcb);
		if(!(getModel() instanceof EObject)) {
			throw new IllegalArgumentException("Model element must be an instance of an EObject.");
		}
	}

	/** Returns the {@link EObject} model element. */
	protected EObject getEObject() {
		// wild cast works: see constructor check with exception
		return (EObject)getModel();
	}

	/** {@inheritDoc} */
	@Override
	public List<MenuItem> contextMenuContributions(Node node, DiagramCoordinate diagramLocation) {
		// outer-most grid areas of the diagram are inaccessible
		DiagramViewerFeatures features = getViewer().getFeatures();
		double x = max(features.getHorizontalSpacing(), diagramLocation.getX());
		double y = max(features.getVerticalSpacing(), diagramLocation.getY());

		// wild cast works: see constructor exception
		// EObject modelParent = (EObject)getModel();
		// IEclipseContext subContext =
		// createSubContextWithCompositionLocation(context, modelParent, x, y);
		// IContextMenuService icms = context.get(IContextMenuService.class);
		// return icms.createDefaultContextMenu(subContext);
		// TODO:
		return Collections.emptyList();
	}

	/** The model change listener list. */
	private final List<IModelChangeListener> changeListeners =
			new LinkedList<IModelChangeListener>();

	/** {@inheritDoc} */
	@Override
	public void addListener(IModelChangeListener l) {
		if(changeListeners.isEmpty()) {
			connectToModel();
		}
		if(!changeListeners.contains(l)) {
			changeListeners.add(l);
		}
	}

	/** Called when sub-class should register with the underlying {@link EObject}. */
	protected abstract void connectToModel();

	/** {@inheritDoc} */
	@Override
	public void removeListener(IModelChangeListener l) {
		changeListeners.remove(l);
		if(changeListeners.isEmpty()) {
			disconnectFromModel();
		}
	}

	/** Called when sub-class should unregister from the underlying {@link EObject}. */
	protected abstract void disconnectFromModel();

	/** {@inheritDoc} */
	@Override
	public void fireUpdateComplete() {
		EObject eo = getEObject();
		for(IModelChangeListener mcl : changeListeners) {
			mcl.changed(eo);
		}
	}
}
