/*******************************************************************************
 * Copyright (c) 2017, 2018 fortiss GmbH. 
 * 
 * This program and the accompanying materials are made available under the
 * terms of the Eclipse Public License v2.0 which is available at
 * http://www.eclipse.org/legal/epl-v20.html
 * 
 * SPDX-License-Identifier: EPL-2.0
 * 
 * Contributors:
 *     Florian Hoelzl (fortiss GmbH) - initial implementation
 *
 *******************************************************************************/
package org.fortiss.tooling.base.ui.editor.fx;

import java.util.LinkedList;
import java.util.List;

import org.eclipse.emf.common.notify.Notification;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.systemfocus.kernel.common.ui.javafx.lwfxef.controller.base.DelegatingContentAnchorageController;
import org.eclipse.systemfocus.kernel.common.ui.javafx.lwfxef.model.IModelChangeListener;
import org.eclipse.systemfocus.kernel.common.ui.javafx.lwfxef.model.IModelChangeProvider;
import org.eclipse.systemfocus.kernel.common.ui.javafx.lwfxef.mvc.IContentAnchorageMVCBundle;
import org.fortiss.tooling.base.model.layout.ILayoutedModelElement;
import org.fortiss.tooling.base.utils.LayoutModelElementAdapter;

/**
 * {@link DelegatingContentAnchorageController} for
 * {@link org.fortiss.tooling.base.model.layout.ILayoutedModelElement}s.
 */
public class LayoutedModelElementBasedContentAnchorageController
		extends DelegatingContentAnchorageController implements IModelChangeProvider {
	/** Layout model adapter used for listening to EMF model changes. */
	private final LayoutModelElementAdapter layoutAdapter;

	/** Constructor. */
	public LayoutedModelElementBasedContentAnchorageController(IContentAnchorageMVCBundle mvcb) {
		super(mvcb);
		if(!(mvcb.getModel() instanceof ILayoutedModelElement)) {
			throw new IllegalArgumentException(
					"Model element must be an instance of an ILayoutedModelElement.");
		}
		ILayoutedModelElement lme = (ILayoutedModelElement)mvcb.getModel();
		layoutAdapter = new LayoutModelElementAdapter(lme) {
			@Override
			public void layoutChanged(Notification notification) {
				fireUpdateComplete();
			}
		};
	}

	/** Returns the model element. */
	public ILayoutedModelElement getLayoutedME() {
		return (ILayoutedModelElement)getModel();
	}

	/** {@inheritDoc} */
	@Override
	public void delete() {
		// TODO
	}

	// /** {@inheritDoc} */
	// @Override
	// protected void link(IMVCBundle startBundle, DiagramCoordinate startLocation,
	// IMVCBundle endBundle, DiagramCoordinate endLocation) {
	// createLinkByCommand(startBundle, startLocation, endBundle, endLocation,
	// (EObject)getViewer().getRootElement(), context);
	// }
	//
	// /** {@inheritDoc} */
	// @Override
	// public boolean isPossibleLinkTarget() {
	// return true;
	// }
	//
	// /** {@inheritDoc} */
	// @Override
	// public MVCBundleTag getLinkTargetEffectForNewLinkFrom(IMVCBundle linkStartBundle) {
	// return getLinkTargetEffect(linkStartBundle, getLayoutedME(),
	// (EObject)getViewer().getRootElement(), context);
	// }

	/** The model change listener list. */
	private final List<IModelChangeListener> changeListeners =
			new LinkedList<IModelChangeListener>();

	/** {@inheritDoc} */
	@Override
	public void addListener(IModelChangeListener l) {
		if(changeListeners.isEmpty()) {
			connectToModel();
		}
		if(!changeListeners.contains(l)) {
			changeListeners.add(l);
		}
	}

	/** Called when sub-class should register with the underlying {@link EObject}. */
	protected void connectToModel() {
		getLayoutedME().eAdapters().add(layoutAdapter);
	}

	/** {@inheritDoc} */
	@Override
	public void removeListener(IModelChangeListener l) {
		changeListeners.remove(l);
		if(changeListeners.isEmpty()) {
			disconnectFromModel();
		}
	}

	/** Called when sub-class should unregister from the underlying {@link EObject}. */
	protected void disconnectFromModel() {
		getLayoutedME().eAdapters().remove(layoutAdapter);
	}

	/** {@inheritDoc} */
	@Override
	public void fireUpdateComplete() {
		EObject eo = getLayoutedME();
		for(IModelChangeListener mcl : changeListeners) {
			mcl.changed(eo);
		}
	}
}
