/*******************************************************************************
 * Copyright (c) 2017, 2018 fortiss GmbH. 
 * 
 * This program and the accompanying materials are made available under the
 * terms of the Eclipse Public License v2.0 which is available at
 * http://www.eclipse.org/legal/epl-v20.html
 * 
 * SPDX-License-Identifier: EPL-2.0
 * 
 * Contributors:
 *     Florian Hoelzl (fortiss GmbH) - initial implementation
 *
 *******************************************************************************/
package org.fortiss.tooling.base.ui.editor.fx;

import java.util.LinkedList;
import java.util.List;

import org.eclipse.emf.common.notify.Notification;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.systemfocus.kernel.common.ui.javafx.lwfxef.FeedbackChange;
import org.eclipse.systemfocus.kernel.common.ui.javafx.lwfxef.controller.base.MoveControllerBase;
import org.eclipse.systemfocus.kernel.common.ui.javafx.lwfxef.model.IModelChangeListener;
import org.eclipse.systemfocus.kernel.common.ui.javafx.lwfxef.model.IModelChangeProvider;
import org.eclipse.systemfocus.kernel.common.ui.javafx.lwfxef.mvc.IMVCBundle;
import org.fortiss.tooling.base.model.layout.ILayoutedModelElement;
import org.fortiss.tooling.base.utils.LayoutModelElementAdapter;

/** {@link MoveControllerBase} for {@link ILayoutedModelElement}s. */
public class LayoutedModelElementBasedDiagramAnchorageController extends MoveControllerBase
		implements IModelChangeProvider {
	/** Layout model adapter used for listening to EMF model changes. */
	private final LayoutModelElementAdapter layoutAdapter;

	/** Constructor. */
	public LayoutedModelElementBasedDiagramAnchorageController(IMVCBundle mvcb) {
		super(mvcb);
		if(!(mvcb.getModel() instanceof ILayoutedModelElement)) {
			throw new IllegalArgumentException(
					"Model element must be an instance of an ILayoutedModelElement.");
		}
		ILayoutedModelElement lme = (ILayoutedModelElement)mvcb.getModel();
		layoutAdapter = new LayoutModelElementAdapter(lme) {
			@Override
			public void layoutChanged(Notification notification) {
				fireUpdateComplete();
			}
		};
	}

	/** Returns the model element. */
	public ILayoutedModelElement getLayoutedME() {
		return (ILayoutedModelElement)getModel();
	}

	/** {@inheritDoc} */
	@Override
	public void move(FeedbackChange delta) {
		// ILayoutedModelElement model = getLayoutedME();
		// Point pos = getPosition(model);
		// int oldX = pos.getX();
		// int oldY = pos.getY();
		// DiagramViewerFeatures features = getViewer().getFeatures();
		// int newX = (int)max(oldX + delta.getDeltaX(), features.getHorizontalSpacing());
		// int newY = (int)max(oldY + delta.getDeltaY(), features.getVerticalSpacing());
		// context.get(IPersistencyService.class).runAsCommand(getModelRoot(model),
		// new AbstractCommand() {
		// @Override
		// public boolean canExecute() {
		// return true;
		// }
		//
		// @Override
		// public void redo() {
		// execute();
		// }
		//
		// @Override
		// public void execute() {
		// setPosition(model, newX, newY);
		// }
		// });
		// setPosition(model, newX, newY);
	}

	/** {@inheritDoc} */
	@Override
	public void delete() {
		// deleteByService(getLayoutedME(), context);
	}

	// /** {@inheritDoc} */
	// @Override
	// protected void link(IMVCBundle startBundle, DiagramCoordinate startLocation,
	// IMVCBundle endBundle, DiagramCoordinate endLocation) {
	// createLinkByCommand(startBundle, startLocation, endBundle, endLocation,
	// (EObject)getViewer().getRootElement(), context);
	// }
	//
	// /** {@inheritDoc} */
	// @Override
	// public boolean isPossibleLinkTarget() {
	// return true;
	// }
	//
	// /** {@inheritDoc} */
	// @Override
	// public MVCBundleTag getLinkTargetEffectForNewLinkFrom(IMVCBundle linkStartBundle) {
	// return getLinkTargetEffect(linkStartBundle, getLayoutedME(),
	// (EObject)getViewer().getRootElement(), context);
	// }

	/** The model change listener list. */
	private final List<IModelChangeListener> changeListeners =
			new LinkedList<IModelChangeListener>();

	/** {@inheritDoc} */
	@Override
	public void addListener(IModelChangeListener l) {
		if(changeListeners.isEmpty()) {
			connectToModel();
		}
		if(!changeListeners.contains(l)) {
			changeListeners.add(l);
		}
	}

	/** Called when sub-class should register with the underlying {@link EObject}. */
	protected void connectToModel() {
		getLayoutedME().eAdapters().add(layoutAdapter);
	}

	/** {@inheritDoc} */
	@Override
	public void removeListener(IModelChangeListener l) {
		changeListeners.remove(l);
		if(changeListeners.isEmpty()) {
			disconnectFromModel();
		}
	}

	/** Called when sub-class should unregister from the underlying {@link EObject}. */
	protected void disconnectFromModel() {
		getLayoutedME().eAdapters().remove(layoutAdapter);
	}

	/** {@inheritDoc} */
	@Override
	public void fireUpdateComplete() {
		EObject eo = getLayoutedME();
		for(IModelChangeListener mcl : changeListeners) {
			mcl.changed(eo);
		}
	}
}
