/*-------------------------------------------------------------------------+
| Copyright 2019 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.tooling.kernel.ui.extension.base;

import org.eclipse.emf.ecore.EObject;
import org.eclipse.systemfocus.kernel.common.ui.javafx.lwfxef.DiagramViewer;
import org.eclipse.systemfocus.kernel.common.ui.javafx.lwfxef.change.Change;
import org.eclipse.systemfocus.kernel.common.ui.javafx.lwfxef.controller.IControllerFactory;
import org.eclipse.systemfocus.kernel.common.ui.javafx.lwfxef.model.IModelFactory;
import org.eclipse.systemfocus.kernel.common.ui.javafx.lwfxef.visual.IVisualFactory;

import javafx.scene.Parent;

/**
 * Base implementation of model element editors which are using LWFXEF DiaViewer.
 * 
 * @author hoelzlf
 */
public abstract class LWFXEFEditorBase<T extends EObject> extends FXEditorBase<T> {
	/** The diagram viewer. */
	protected DiagramViewer viewer;

	/** {@inheritDoc} */
	@Override
	protected final Parent createSceneRoot() {
		viewer = new DiagramViewer(createModelFactory(), createVisualFactory(),
				createControllerFactory(), cb -> {
					modelSelected(cb);
				}, chg -> {
					applyModelChange(chg);
				});
		customizeViewer();
		return viewer.getVisualNode();
	}

	/** Customize the viewer after it is created. The default implementation changes nothing. */
	protected void customizeViewer() {
		// the default does nothing
	}

	/** Create the {@link IModelFactory}. */
	protected abstract IModelFactory createModelFactory();

	/** Create the {@link IVisualFactory}. */
	protected abstract IVisualFactory createVisualFactory();

	/** Create the {@link IControllerFactory}. */
	protected abstract IControllerFactory createControllerFactory();

	/** Called when a model element is selected in the diagram viewer. */
	protected abstract void modelSelected(Object model);

	/** Called when some change to the model happens. */
	protected abstract void applyModelChange(Change chg);
}
