/*******************************************************************************
 * Copyright (c) 2016, 2018 fortiss GmbH. 
 * 
 * This program and the accompanying materials are made available under the
 * terms of the Eclipse Public License v2.0 which is available at
 * http://www.eclipse.org/legal/epl-v20.html
 * 
 * SPDX-License-Identifier: EPL-2.0
 * 
 * Contributors:
 *     Florian Hoelzl (fortiss GmbH) - initial implementation
 *
 *******************************************************************************/
package org.eclipse.systemfocus.kernel.common.ui.javafx.lwfxef.controller.rectangular;

import static org.eclipse.systemfocus.kernel.common.ui.javafx.lwfxef.visual.rectangular.RectangularBorderLocation.getClosestLocationOnBounds;

import org.eclipse.systemfocus.kernel.common.ui.javafx.lwfxef.DiagramCoordinate;
import org.eclipse.systemfocus.kernel.common.ui.javafx.lwfxef.controller.IDragController;
import org.eclipse.systemfocus.kernel.common.ui.javafx.lwfxef.controller.base.AnchorageContentControllerBase;
import org.eclipse.systemfocus.kernel.common.ui.javafx.lwfxef.controller.base.ContentAnchorageMoveControllerBase;
import org.eclipse.systemfocus.kernel.common.ui.javafx.lwfxef.mvc.IContentAnchorageMVCBundle;
import org.eclipse.systemfocus.kernel.common.ui.javafx.lwfxef.mvc.IContentMVCBundle;
import org.eclipse.systemfocus.kernel.common.ui.javafx.lwfxef.visual.rectangular.RectangularBorderLocation;

import javafx.geometry.Dimension2D;
import javafx.geometry.Point2D;
import javafx.geometry.Rectangle2D;
import javafx.geometry.Side;

/** Base class for controllers with content visuals represented by rectangles. */
public abstract class RectangularResizableContentControllerBase
		extends AnchorageContentControllerBase {
	/** Constructor. */
	public RectangularResizableContentControllerBase(IContentMVCBundle mvcb) {
		super(mvcb);
	}

	/** {@inheritDoc} */
	@Override
	protected IDragController createAnchorageMoveController(IContentAnchorageMVCBundle anchorage) {
		return new RectangularAnchorageMoveController(anchorage);
	}

	/** Move controller for anchorages moving on the rectangle border. */
	private class RectangularAnchorageMoveController extends ContentAnchorageMoveControllerBase {
		/** Constructor. */
		public RectangularAnchorageMoveController(IContentAnchorageMVCBundle anchorage) {
			super(anchorage);
		}

		/** Returns the location clamped to the border of the parent visual. */
		private RectangularBorderLocation getLocationOnBorder(IContentAnchorageMVCBundle anchorage,
				Point2D delta) {
			Rectangle2D contentBounds = getVisual().getModelBounds();
			Rectangle2D anchorageBounds = anchorage.getVisual().getModelBounds();
			Dimension2D span =
					new Dimension2D(anchorageBounds.getWidth(), anchorageBounds.getHeight());
			DiagramCoordinate p = new DiagramCoordinate(anchorageBounds.getMinX() + delta.getX(),
					anchorageBounds.getMinY() + delta.getY());
			RectangularBorderLocation rbl = getClosestLocationOnBounds(p, contentBounds, span);
			return rbl;
		}

		/** {@inheritDoc} */
		@Override
		protected Point2D getAnchorageFeedbackLocation(DiagramCoordinate delta) {
			RectangularBorderLocation rbl = getLocationOnBorder(anchorage, delta);
			return rbl.getLocation();
		}

		/** {@inheritDoc} */
		@Override
		protected void moveAnchorage(DiagramCoordinate delta) {
			RectangularBorderLocation rbl = getLocationOnBorder(anchorage, delta);
			moveAnchorageToSideOffset(anchorage, rbl.getSide(), rbl.getOffset());
		}
	}

	/** Sets the side and offset of the given anchorage when move operation has completed. */
	protected abstract void moveAnchorageToSideOffset(IContentAnchorageMVCBundle anchorage,
			Side side, double offset);
}
